@@ -1,13 +1,12 @@
 //! A `Compilation` contains the compiled function bodies for a WebAssembly
 //! module.
 
+use crate::error::Result;
 use crate::prelude::*;
-use crate::{obj, Tunables};
 use crate::{
-    BuiltinFunctionIndex, DefinedFuncIndex, FlagValue, FuncIndex, FunctionLoc, ObjectKind,
-    PrimaryMap, StaticModuleIndex, TripleExt, WasmError, WasmFuncType,
+    DefinedFuncIndex, FlagValue, FuncKey, FunctionLoc, ObjectKind, PrimaryMap, StaticModuleIndex,
+    TripleExt, Tunables, WasmError, WasmFuncType, obj,
 };
-use anyhow::Result;
 use object::write::{Object, SymbolId};
 use object::{Architecture, BinaryFormat, FileFlags};
 use std::any::Any;
@@ -17,6 +16,7 @@ use std::path;
 use std::sync::Arc;
 
 mod address_map;
+mod frame_table;
 mod module_artifacts;
 mod module_environ;
 mod module_types;
@@ -24,6 +24,7 @@ mod stack_maps;
 mod trap_encoding;
 
 pub use self::address_map::*;
+pub use self::frame_table::*;
 pub use self::module_artifacts::*;
 pub use self::module_environ::*;
 pub use self::module_types::*;
@@ -62,7 +63,7 @@ impl From<WasmError> for CompileError {
 }
 
 impl core::error::Error for CompileError {
-    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+    fn source(&self) -> Option<&(dyn core::error::Error + 'static)> {
         match self {
             CompileError::Wasm(e) => Some(e),
             _ => None,
@@ -70,21 +71,6 @@ impl core::error::Error for CompileError {
     }
 }
 
-/// What relocations can be applied against.
-///
-/// Each wasm function may refer to various other `RelocationTarget` entries.
-#[derive(Copy, Clone, Debug, PartialEq, Eq)]
-pub enum RelocationTarget {
-    /// This is a reference to another defined wasm function in the same module.
-    Wasm(FuncIndex),
-    /// This is a reference to a trampoline for a builtin function.
-    Builtin(BuiltinFunctionIndex),
-    /// A compiler-generated libcall.
-    HostLibcall(obj::LibCall),
-    /// A pulley->host call from the interpreter.
-    PulleyHostcall(u32),
-}
-
 /// Implementation of an incremental compilation's key/value cache store.
 ///
 /// In theory, this could just be Cranelift's `CacheKvStore` trait, but it is not as we want to
@@ -93,7 +79,7 @@ pub enum RelocationTarget {
 pub trait CacheStore: Send + Sync + std::fmt::Debug {
     /// Try to retrieve an arbitrary cache key entry, and returns a reference to bytes that were
     /// inserted via `Self::insert` before.
-    fn get(&self, key: &[u8]) -> Option<Cow<[u8]>>;
+    fn get(&self, key: &[u8]) -> Option<Cow<'_, [u8]>>;
 
     /// Given an arbitrary key and bytes, stores them in the cache.
     ///
@@ -144,6 +130,9 @@ pub trait CompilerBuilder: Send + Sync + fmt::Debug {
     /// Set the tunables for this compiler.
     fn set_tunables(&mut self, tunables: Tunables) -> Result<()>;
 
+    /// Get the tunables used by this compiler.
+    fn tunables(&self) -> Option<&Tunables>;
+
     /// Builds a new [`Compiler`] object from this configuration.
     fn build(&self) -> Result<Box<dyn Compiler>>;
 
@@ -182,7 +171,7 @@ pub enum SettingKind {
 pub struct CompiledFunctionBody {
     /// The code. This is whatever type the `Compiler` implementation wants it
     /// to be, we just shepherd it around.
-    pub code: Box<dyn Any + Send>,
+    pub code: Box<dyn Any + Send + Sync>,
     /// Whether the compiled function needs a GC heap to run; that is, whether
     /// it reads a struct field, allocates, an array, or etc...
     pub needs_gc_heap: bool,
@@ -190,7 +179,69 @@ pub struct CompiledFunctionBody {
 
 /// An implementation of a compiler which can compile WebAssembly functions to
 /// machine code and perform other miscellaneous tasks needed by the JIT runtime.
+///
+/// The diagram below depicts typical usage of this trait:
+///
+/// ```text
+///                     +------+
+///                     | Wasm |
+///                     +------+
+///                        |
+///                        |
+///           Compiler::compile_function()
+///                        |
+///                        |
+///                        V
+///             +----------------------+
+///             | CompiledFunctionBody |
+///             +----------------------+
+///               |                  |
+///               |                  |
+///               |                When
+///               |       Compiler::inlining_compiler()
+///               |               is some
+///               |                  |
+///             When                 |
+/// Compiler::inlining_compiler()    |-----------------.
+///             is none              |                 |
+///               |                  |                 |
+///               |           Optionally call          |
+///               |        InliningCompiler::inline()  |
+///               |                  |                 |
+///               |                  |                 |
+///               |                  |-----------------'
+///               |                  |
+///               |                  |
+///               |                  V
+///               |     InliningCompiler::finish_compiling()
+///               |                  |
+///               |                  |
+///               |------------------'
+///               |
+///               |
+///   Compiler::append_code()
+///               |
+///               |
+///               V
+///           +--------+
+///           | Object |
+///           +--------+
+/// ```
 pub trait Compiler: Send + Sync {
+    /// Get this compiler's inliner.
+    ///
+    /// Consumers of this trait **must** check for when when this method returns
+    /// `Some(_)`, and **must** call `InliningCompiler::finish_compiling` on all
+    /// `CompiledFunctionBody`s produced by this compiler in that case before
+    /// passing the the compiled functions to `Compiler::append_code`, even if
+    /// the consumer does not actually intend to do any inlining. This allows
+    /// implementations of the trait to only translate to an internal
+    /// representation in `Compiler::compile_*` methods so that they can then
+    /// perform inlining afterwards if the consumer desires, and then finally
+    /// proceed with compilng that internal representation to native code in
+    /// `InliningCompiler::finish_compiling`.
+    fn inlining_compiler(&self) -> Option<&dyn InliningCompiler>;
+
     /// Compiles the function `index` within `translation`.
     ///
     /// The body of the function is available in `data` and configuration
@@ -199,9 +250,10 @@ pub trait Compiler: Send + Sync {
     fn compile_function(
         &self,
         translation: &ModuleTranslation<'_>,
-        index: DefinedFuncIndex,
+        key: FuncKey,
         data: FunctionBodyData<'_>,
         types: &ModuleTypesBuilder,
+        symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError>;
 
     /// Compile a trampoline for an array-call host function caller calling the
@@ -213,7 +265,8 @@ pub trait Compiler: Send + Sync {
         &self,
         translation: &ModuleTranslation<'_>,
         types: &ModuleTypesBuilder,
-        index: DefinedFuncIndex,
+        key: FuncKey,
+        symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError>;
 
     /// Compile a trampoline for a Wasm caller calling a array callee with the
@@ -224,6 +277,8 @@ pub trait Compiler: Send + Sync {
     fn compile_wasm_to_array_trampoline(
         &self,
         wasm_func_ty: &WasmFuncType,
+        key: FuncKey,
+        symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError>;
 
     /// Creates a trampoline that can be used to call Wasmtime's implementation
@@ -237,7 +292,8 @@ pub trait Compiler: Send + Sync {
     /// call.
     fn compile_wasm_to_builtin(
         &self,
-        index: BuiltinFunctionIndex,
+        key: FuncKey,
+        symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError>;
 
     /// Returns the list of relocations required for a function from one of the
@@ -245,7 +301,7 @@ pub trait Compiler: Send + Sync {
     fn compiled_function_relocation_targets<'a>(
         &'a self,
         func: &'a dyn Any,
-    ) -> Box<dyn Iterator<Item = RelocationTarget> + 'a>;
+    ) -> Box<dyn Iterator<Item = FuncKey> + 'a>;
 
     /// Appends a list of compiled functions to an in-memory object.
     ///
@@ -277,8 +333,8 @@ pub trait Compiler: Send + Sync {
     fn append_code(
         &self,
         obj: &mut Object<'static>,
-        funcs: &[(String, Box<dyn Any + Send>)],
-        resolve_reloc: &dyn Fn(usize, RelocationTarget) -> usize,
+        funcs: &[(String, FuncKey, Box<dyn Any + Send + Sync>)],
+        resolve_reloc: &dyn Fn(usize, FuncKey) -> usize,
     ) -> Result<Vec<(SymbolId, FunctionLoc)>>;
 
     /// Creates a new `Object` file which is used to build the results of a
@@ -308,7 +364,7 @@ pub trait Compiler: Send + Sync {
             Pulley32 | Pulley32be => (Architecture::Riscv64, obj::EF_WASMTIME_PULLEY32),
             Pulley64 | Pulley64be => (Architecture::Riscv64, obj::EF_WASMTIME_PULLEY64),
             architecture => {
-                anyhow::bail!("target architecture {:?} is unsupported", architecture,);
+                anyhow::bail!("target architecture {architecture:?} is unsupported");
             }
         };
         let mut obj = Object::new(
@@ -388,7 +444,7 @@ pub trait Compiler: Send + Sync {
         get_func: &'a dyn Fn(
             StaticModuleIndex,
             DefinedFuncIndex,
-        ) -> (SymbolId, &'a (dyn Any + Send)),
+        ) -> (SymbolId, &'a (dyn Any + Send + Sync)),
         dwarf_package_bytes: Option<&'a [u8]>,
         tunables: &'a Tunables,
     ) -> Result<()>;
@@ -401,3 +457,36 @@ pub trait Compiler: Send + Sync {
         None
     }
 }
+
+/// An inlining compiler.
+pub trait InliningCompiler: Sync + Send {
+    /// Enumerate the function calls that the given `func` makes.
+    fn calls(&self, func: &CompiledFunctionBody, calls: &mut IndexSet<FuncKey>) -> Result<()>;
+
+    /// Get the abstract size of the given function, for the purposes of
+    /// inlining heuristics.
+    fn size(&self, func: &CompiledFunctionBody) -> u32;
+
+    /// Process this function for inlining.
+    ///
+    /// Implementations should call `get_callee` for each of their direct
+    /// function call sites and if `get_callee` returns `Some(_)`, they should
+    /// inline the given function body into that call site.
+    fn inline<'a>(
+        &self,
+        func: &mut CompiledFunctionBody,
+        get_callee: &'a mut dyn FnMut(FuncKey) -> Option<&'a CompiledFunctionBody>,
+    ) -> Result<()>;
+
+    /// Finish compiling the given function.
+    ///
+    /// This method **must** be called before passing the
+    /// `CompiledFunctionBody`'s contents to `Compiler::append_code`, even if no
+    /// inlining was performed.
+    fn finish_compiling(
+        &self,
+        func: &mut CompiledFunctionBody,
+        input: Option<wasmparser::FunctionBody<'_>>,
+        symbol: &str,
+    ) -> Result<()>;
+}
