@@ -195,8 +195,7 @@
 //!
 //! * `cache` - Enabled by default, this feature adds support for wasmtime to
 //!   perform internal caching of modules in a global location. This must still
-//!   be enabled explicitly through [`Config::cache_config_load`] or
-//!   [`Config::cache_config_load_default`].
+//!   be enabled explicitly through [`Config::cache`].
 //!
 //! * `wat` - Enabled by default, this feature adds support for accepting the
 //!   text format of WebAssembly in [`Module::new`] and
@@ -280,12 +279,12 @@
 #![deny(missing_docs)]
 #![doc(test(attr(deny(warnings))))]
 #![doc(test(attr(allow(dead_code, unused_variables, unused_mut))))]
-#![cfg_attr(docsrs, feature(doc_auto_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 // NB: this list is currently being burned down to remove all features listed
 // here to get warnings in all configurations of Wasmtime.
 #![cfg_attr(
     any(not(feature = "runtime"), not(feature = "std")),
-    allow(dead_code, unused_imports)
+    expect(dead_code, unused_imports, reason = "list not burned down yet")
 )]
 // Allow broken links when the default features is disabled because most of our
 // documentation is written for the "one build" of the `main` branch which has
@@ -293,16 +292,26 @@
 // and will prevent the doc build from failing.
 #![cfg_attr(feature = "default", warn(rustdoc::broken_intra_doc_links))]
 #![no_std]
-#![expect(clippy::allow_attributes_without_reason, reason = "crate not migrated")]
+// Wasmtime liberally uses #[cfg]'d definitions of structures to uninhabited
+// types to reduce the total amount of #[cfg], but rustc warns that much usage
+// of these structures, rightfully, leads to unreachable code. This unreachable
+// code is only conditional, however, so it's generally just annoying to deal
+// with. Disable the `unreachable_code` lint in situations like this when some
+// major features are disabled. If all the features are enabled, though, we
+// still want to get warned about this.
+#![cfg_attr(
+    any(not(feature = "threads"), not(feature = "gc",)),
+    allow(unreachable_code, reason = "see comment")
+)]
 
-#[cfg(any(feature = "std", unix, windows))]
+#[cfg(feature = "std")]
 #[macro_use]
 extern crate std;
 extern crate alloc;
 
 pub(crate) mod prelude {
     pub use crate::{Error, Result};
-    pub use anyhow::{anyhow, bail, ensure, format_err, Context};
+    pub use anyhow::{Context, anyhow, bail, ensure, format_err};
     pub use wasmtime_environ::prelude::*;
 }
 
@@ -333,7 +342,7 @@ pub(crate) use hashbrown::{hash_map, hash_set};
 #[macro_export]
 macro_rules! map_maybe_uninit {
     ($maybe_uninit:ident $($field:tt)*) => ({
-        #[allow(unused_unsafe)]
+        #[allow(unused_unsafe, reason = "macro-generated code")]
         {
             unsafe {
                 use $crate::MaybeUninitExt;
@@ -353,10 +362,12 @@ macro_rules! map_maybe_uninit {
 pub trait MaybeUninitExt<T> {
     /// Maps `MaybeUninit<T>` to `MaybeUninit<U>` using the closure provided.
     ///
-    /// Note that this is `unsafe` as there is no guarantee that `U` comes from
-    /// `T`.
+    /// # Safety
+    ///
+    /// Requires that `*mut U` is a field projection from `*mut T`. Use
+    /// `map_maybe_uninit!` above instead.
     unsafe fn map<U>(&mut self, f: impl FnOnce(*mut T) -> *mut U)
-        -> &mut core::mem::MaybeUninit<U>;
+    -> &mut core::mem::MaybeUninit<U>;
 }
 
 impl<T> MaybeUninitExt<T> for core::mem::MaybeUninit<T> {
@@ -365,7 +376,10 @@ impl<T> MaybeUninitExt<T> for core::mem::MaybeUninit<T> {
         f: impl FnOnce(*mut T) -> *mut U,
     ) -> &mut core::mem::MaybeUninit<U> {
         let new_ptr = f(self.as_mut_ptr());
-        core::mem::transmute::<*mut U, &mut core::mem::MaybeUninit<U>>(new_ptr)
+        // SAFETY: the memory layout of these two types are the same, and
+        // asserting that it's a safe reference with the same lifetime as `self`
+        // is a requirement of this function itself.
+        unsafe { core::mem::transmute::<*mut U, &mut core::mem::MaybeUninit<U>>(new_ptr) }
     }
 }
 
@@ -391,7 +405,6 @@ mod sync_std;
 #[cfg(feature = "std")]
 use sync_std as sync;
 
-#[cfg_attr(feature = "std", allow(dead_code))]
 mod sync_nostd;
 #[cfg(not(feature = "std"))]
 use sync_nostd as sync;
