@@ -4,15 +4,16 @@
 //! `Option<Thing>` is also zero-sized so there should be no runtime cost for
 //! having these structures plumbed around.
 
-use crate::runtime::vm::VMOpaqueContext;
-use crate::runtime::Uninhabited;
+use crate::runtime::vm::{VMContext, VMOpaqueContext};
 use crate::{Engine, ValRaw};
+use core::convert::Infallible;
 use core::marker;
 use core::mem;
 use core::ptr::NonNull;
+use wasmtime_unwinder::Unwind;
 
 pub struct Interpreter {
-    empty: Uninhabited,
+    empty: Infallible,
 }
 
 const _: () = assert!(mem::size_of::<Interpreter>() == 0);
@@ -26,10 +27,14 @@ impl Interpreter {
     pub fn as_interpreter_ref(&mut self) -> InterpreterRef<'_> {
         match self.empty {}
     }
+
+    pub fn unwinder(&self) -> &'static dyn Unwind {
+        match self.empty {}
+    }
 }
 
 pub struct InterpreterRef<'a> {
-    empty: Uninhabited,
+    empty: Infallible,
     _marker: marker::PhantomData<&'a mut Interpreter>,
 }
 
@@ -41,9 +46,18 @@ impl InterpreterRef<'_> {
         self,
         _bytecode: NonNull<u8>,
         _callee: NonNull<VMOpaqueContext>,
-        _caller: NonNull<VMOpaqueContext>,
+        _caller: NonNull<VMContext>,
         _args_and_results: NonNull<[ValRaw]>,
     ) -> bool {
         match self.empty {}
     }
+
+    pub(crate) unsafe fn resume_to_exception_handler(
+        &mut self,
+        _handler: &wasmtime_unwinder::Handler,
+        _payload1: usize,
+        _payload2: usize,
+    ) {
+        match self.empty {}
+    }
 }
